/*

Author: Aiven Wolf
Date: 21.2. 2025
email: technik@lcd-module.de


  //Getting Started with Modbus TCP on Opta™
  Name: Opta_Client
  Purpose: Writes Coil and Holding Register values; Reads Coil, Discrete Input, Holding Registers, and Input Register values.

  @author Arduino
*/
#include <WiFi.h>            //includes WiFi library
#include <ArduinoModbus.h>   //includes Modbus Library
#include <SPI.h>      //includes SPI library
#include <Ethernet.h>         //includes Ethernet library

WiFiClient  wifiClient;           //Network acsess 
ModbusTCPClient modbusTCPClient(wifiClient);    //is Modbus Master

IPAddress ip(192, 168, 2, 10);       //sets IPAddress
IPAddress IP_HMI(192, 168, 2, 20);   //sets IPAddress from HMI display




#define SECRET_SSID "DV_PLCTraining"         //defines SSID
#define SECRET_PASS "DV!PLCTraining82205"   //defines WiFi password

char ssid[] = SECRET_SSID;    //Networkname
char pass[] = SECRET_PASS;    //Networkpassword

constexpr auto modbus_adress = 42;               //modbus adress
constexpr auto baudrate { 115200 };             //sets baudrate

// Calculate preDelay and postDelay in microseconds as per Modbus TCP Specification
// MODBUS over serial line specification and implementation guide V1.02
// Paragraph 2.5.1.1 MODBUS Message TCP Framing
// https://modbus.org/docs/Modbus_over_serial_line_V1_02.pdf
constexpr auto bitduration { 1.f / baudrate };
constexpr auto preDelayBR { bitduration * 9.6f * 3.5f * 1e6 };
constexpr auto postDelayBR { bitduration * 9.6f * 3.5f * 1e6 };
// constexpr auto preDelayBR { bitduration * 10.0f * 3.5f * 1e6 };

int status = WL_IDLE_STATUS;
bool in_switch = 0;  //holds the value for Input
bool out_light = 0;  //holds the value for output
bool modbus_on = 1;  //bool to interrupt modbus communication
bool last_user_state = 1;

int in_analog = 0;  //holds the value for analoginput

void setup() {
    Serial.begin(9600);
    while (!Serial);

    Serial.println("Modbus TCP Master");
    Serial.print("Connecting to ");         //prints in serial monitor
    Serial.println(ssid);
  
    status = WiFi.begin(ssid, pass);
      while (status != WL_CONNECTED) {
        delay(10000);                      //waits 10 seconds to connect again
        Serial.print("Connecting to ");     //prints Connecting to in serial monitor
        Serial.println(ssid);
        status = WiFi.begin(ssid, pass);
  }
    Serial.println("Connected to wifi");    //if connected to WiFi, prints in serial monitor


    pinMode(A0,  INPUT);              //input from pin A0
    pinMode(RELAY1, OUTPUT);       //output on in RELAY1
    pinMode (BTN_USER, INPUT);  //User button is used to interrupt modbus communication
    analogReadResolution(12);
}

void loop() {
  
  if (digitalRead(BTN_USER) == 0 && last_user_state){   //checks button and last user state
  digitalWrite(LED_USER, modbus_on);    //turns on user LED on or off  
  modbus_on = !modbus_on;         //set modbus = 1 to 0 and Modbus = 0 to 1
  last_user_state = 0;           //set last user to 0
  
  if (modbus_on == 0 && modbusTCPClient.connected() == 1)
     modbusTCPClient.stop();
  
  
  }
  if (digitalRead(BTN_USER) == 1 && last_user_state ==0){
  last_user_state = 1; 
  }
  if (modbus_on){
        // Attempt to connect to Modbus TCP server if not already connected
    if (!modbusTCPClient.connected()) {                                   //if modbus is not connected, prints in serial Monitor
        Serial.println("- Attempting to connect to Modbus TCP server...");             //prints in serial monitor

        // Start Modbus TCP client
        if (!modbusTCPClient.begin(IP_HMI, 502)) {                        //if modbus is not connected, print in serial Monitor
            Serial.println("- Failed to connect to Modbus TCP server!");
        } else {
            Serial.println("- Connected to Modbus TCP server!");
        }
      }
      else{
    in_switch = digitalRead(A0);   //reads Input pin A0
    writeCoilValues();           //send input state

    readDiscreteInputValues();          //read display, button state
    digitalWrite(RELAY1, out_light);   //output on pin RELAY1
    digitalWrite(LED_RELAY1, out_light);

    in_analog = analogRead(A1);        //reads analog value (12bit)
    writeHoldingRegisterValues();      //send analog value to display
    Serial.println();
      }
   }
   
   delay(100);    //pause
} 

//Writes Coil values to the server under specified address.
void writeCoilValues() {

    Serial.print("Writing Coil values ... ");
    // Write 10 Coil values to (server) id 42, address 0x00
    
    if (!modbusTCPClient.coilWrite(0x1, in_switch)) {
        Serial.print("failed! ");                      //prints failed in serial monitor
        Serial.println(modbusTCPClient.lastError());
    } else {                                             
        Serial.println("success");          //prints success in serial monitor.
    }
}
     //Reads Discrete Input values from the server under specified address.
void readDiscreteInputValues() {                        //reads input values
    Serial.print("Reading Discrete Input values ... ");  //prints in serial monitor
    int value = modbusTCPClient.discreteInputRead(0x01);   //sets variaable value

    if (value == -1) {                               //checks value
        Serial.print("failed! ");                   //prints in serial monitor
        Serial.println(modbusTCPClient.lastError()); 
    } else {                                         //if value is not -1:
        Serial.println("success");                  //prints success in serial monitor

          out_light = value;           //sets out_light to value (variables)          
            Serial.print(out_light);         
            Serial.print(' ');
        }
        Serial.println();
}

     // Writes Holding Register values to the server under specified address.
void writeHoldingRegisterValues() {
    //Set the Holding Register values to counter
    Serial.print("Writing Holding Registers values ... ");

    // Write 10 coil values to (server) id 42, address 0x00
    modbusTCPClient.beginTransmission(modbus_adress, HOLDING_REGISTERS, 0x1, 2);
        modbusTCPClient.write(in_analog);          
        modbusTCPClient.write(0);

    int ret;                                                       //sets integer variable
    ret = modbusTCPClient.holdingRegisterWrite( 0x1, in_analog);
    if (ret == 1)                                              //checks ret
       ret = modbusTCPClient.holdingRegisterWrite(2, 0);


    if (ret = 0) {                             //checks variable ret
        Serial.print("failed! ");            //prints failed! in serial monitor
        Serial.println(modbusTCPClient.lastError());    
    } else {                                 //if ret is not 0:
        Serial.println("success");              //prints "success" in serial Monitor
    }
}


